/*******************************************************************************/
/*                                                                             */
/*    Copyright (c) 2007 Analog Devices, Inc.  All Rights Reserved.            */
/*    This software is proprietary to Analog Devices, Inc. and its 			   */
/*	  licensors.                                  							   */
/*                                                                             */
/*    CHANGES:  07/02/2007  1.00  - initial public release  				   */
/*																			   */
/*******************************************************************************/

/*********************************************************************

Include files

*********************************************************************/

#include <ccblkfn.h>
#include "main.h"
#include "a1836.h"
#include "system.h"

#define FLOW_1 0x1000
#define SLEN_32 0x1f

/*
	Function:Init_Sport0	
	
	Description:Configure Sport0 for I2S mode, to transmit/receive data 	
				to/from the ADC/DAC.Configure Sport for external clocks and 
				frame syncs.
*/
void Init_DMA_Sport0(char * iRxBuffer1)
{
	*pSPORT0_RCR1 = RFSR;
	*pSPORT0_RCR2 = SLEN_32;
	
	// Enable MCM 8 transmit & receive channels
	*pSPORT0_MRCS0 = 0x000000FF;
	
	// Set MCM configuration register and enable MCM mode
	*pSPORT0_MCMC1 = 0x0000;
	*pSPORT0_MCMC2 = 0x101c;
	
	// Configure DMA3
	// 32-bit transfers, Interrupt on completion, Autobuffer mode
	*pDMA2_0_CONFIG = WNR |WDSIZE_32| FLOW_1 | DI_EN;
	// Start address of data buffer
	*pDMA2_0_START_ADDR = iRxBuffer1;
	// DMA loop count
	*pDMA2_0_X_COUNT = 8;
	// DMA loop address increment
	*pDMA2_0_X_MODIFY = 4;
	
}

/*
	Function: void Enable_DMA_Sport0(void)
	
	Description: Enable DMA3, DMA4, Sport0 TX and Sport0 RX
*/
void Enable_DMA_Sport0(void)
{
	// enable DMAs
	*pDMA2_0_CONFIG	= (*pDMA2_0_CONFIG | DMAEN);
	ssync();
	*pSPORT0_RCR1 	= (*pSPORT0_RCR1 | RSPEN);
	ssync();
}

/*
	Function:void Disable_Sport0(void)
	
	Description:
*/
void Disable_Sport0(void)
{
	
	*pSPORT0_RCR1 	= (*pSPORT0_RCR1 & 0x0);
	ssync();
	*pDMA2_0_CONFIG	= (*pDMA2_0_CONFIG & 0x0);
	ssync();
	
}

/*
	Function: Init_Interrupts	
	
	Description: Initialize Interrupt for Sport0 RX
*/
void Init1836(void)
{
	int wait_reset;
	int wait_dma_finish;

	// reset codec
	// set PF15 as output
	*pFIO0_DIR |= (1 << AD1836_RESET_bit); 
	ssync();
	// clear bit to enable AD1836
	*pFIO0_FLAG_S |= (1 << AD1836_RESET_bit);	
	ssync();

	// wait to recover from reset
	for (wait_reset=0; wait_reset<0xf000; wait_reset++);

	// Enable PF4
	*pSPI_FLG = FLS4;
	// Set baud rate SCK = HCLK/(2*SPIBAUD)	
	*pSPI_BAUD = 16;
	// configure spi port
	// SPI DMA write, 16-bit data, MSB first, SPI Master
	*pSPI_CTL = TIMOD_DMA_TX | SIZE | MSTR;
	
	// Set up DMA2 channel 4 to SPI transmit
	*pDMA2_4_PERIPHERAL_MAP = 0x4000;
	
	// Configure DMA2 channel4
	// 16-bit transfers
	*pDMA2_4_CONFIG = WDSIZE_16;
	// Start address of data buffer
	*pDMA2_4_START_ADDR = (void *)sCodec1836TxRegs;
	// DMA inner loop count
	*pDMA2_4_X_COUNT = CODEC_1836_REGS_LENGTH;
	// Inner loop address increment
	*pDMA2_4_X_MODIFY = 2;
	
	// enable DMAs
	*pDMA2_4_CONFIG = (*pDMA2_4_CONFIG | DMAEN);
	// enable spi
	*pSPI_CTL = (*pSPI_CTL | SPE);
	ssync();
	
	// wait until dma transfers for spi are finished 
	for (wait_dma_finish=0; wait_dma_finish<0xaff; wait_dma_finish++);
	
	// disable spi
	*pSPI_CTL = 0x0000;
}
